/*
 *  cubesql_plugin.h
 *
 *  Created by Marco Bambini on 1/3/08.
 *  Copyright 2005-2011 SQLabs. All rights reserved.
 *	
 */

#ifndef _CUBESQL_PLUGIN_H_
#define _CUBESQL_PLUGIN_H_

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef __cplusplus
extern "C" {
#endif

#ifdef WIN32
#define CUBESQL_WINDOWS
#include <Windows.h>
#else
#ifdef __APPLE__
#define CUBESQL_MAC
#include <Carbon/Carbon.h>
#else
#define CUBESQL_LINUX
#endif
#endif
	
#ifdef WIN32
#define CUBESQL_API __declspec(dllexport)
#define DIRREF	HANDLE
#else
#include <dirent.h>
#define CUBESQL_API
#define DIRREF	DIR*
#endif

#define CUBESQL_API_VESION		1
#define CUBESQL_ANY_NUMARGS		-1

#define kFALSE					0
#define kTRUE					1
#define	kSELECT					2
#define kEXECUTE				3
#define CUBESQL_SEND_OK			1
#define CUBESQL_SEND_CURSOR		2
#define CUBESQL_SEND_ERROR		9
#define kCommandIDNone			0
	
// flags for the csql_db_open function
#define CUBESQL_DONT_CREATE			8
#define	CUBESQL_CREATE_IF_NEEDED	9

#define CUBESQL_UTF8			1
#define CUBESQL_UTF16LE			2
#define CUBESQL_UTF16BE			3
#define CUBESQL_UTF16			4    /* Use native byte order */
#define CUBESQL_ANY				5    /* cubesql_plugin_register_function/cubesql_plugin_register_aggregate only */
#define CUBESQL_UTF16_ALIGNED	8    /* cubesql_plugin_register_collation only */

#define CUBESQL_LESS			-1
#define CUBESQL_BIG				1
#define CUBESQL_EQUAL			0

#define CUBESQL_STATIC			(void(*)(void* ))(0)
#define CUBESQL_TRANSIENT		(void(*)(void* ))(-1)
#define CUBESQL_ZEROTERM		(-1)
	
#define CUBESQL_INTEGER			1
#define CUBESQL_FLOAT			2
#define CUBESQL_TEXT			3
#define CUBESQL_BLOB			4
#define CUBESQL_NULL			5
	
#define SET_PLUGIN_NAME(v)			(*name=(char *)v)
#define SET_PLUGIN_VERSION(v)		(*version=(char *)v)
#define SET_PLUGIN_COPYRIGHT(v)		(*copyright=(char *)v)
#define SET_PLUGIN_DESCRIPTION(v)	(*description=(char *)v)
#define SET_PLUGIN_EXTENSIONS(v)	(*extensions=v)
#define SET_PLUGIN_SERIALS(v)		(*serials=v)
	
// privilege codes
enum {
	privilege_None				= 0,
	privilege_Admin				= 3,
	privilege_Privileges		= 4,
	privilege_Preferences		= 5,
	privilege_Databases			= 6,
	privilege_Select			= 7,
	privilege_Insert			= 8,
	privilege_Update			= 9,
	privilege_Delete			= 10,
	privilege_Pragma			= 11,
	privilege_Create			= 12,
	privilege_Drop				= 13,
	privilege_Upload			= 14,
	privilege_Download			= 15,
	privilege_Server			= 16,
	privilege_Restore			= 17,
	privilege_Backup			= 18,
	privilege_Plugin			= 19,
	privilege_DatabaseAdmin		= 20,
	
	// synthetic privileges that never appear in settings db
	privilege_Write				= 100,
	privilege_Read				= 101,
	privilege_ReadWrite			= 102
};	
	
#if defined(_MSC_VER) || defined(__BORLANDC__)
  typedef __int64 cubesql_int64;
#else
  typedef long long int cubesql_int64;
#endif

typedef struct cubesql_context cubesql_context;
typedef struct cubesql_value cubesql_value;
typedef struct cubesql_client cubesql_client;
typedef struct cubesql_db cubesql_db;

typedef struct PluginResolver
{
	// main functions (5)
	int		(*csql_register_sql_function)  (void *pluginref, const char *name, int num_args, int text_encoding, void *user_data, void (*xFunc)(cubesql_context *ctx, int argc, cubesql_value **argv));
	int		(*csql_register_sql_aggregate) (void *pluginref, const char *name, int num_args, int text_encoding, void *user_data, void (*xStep)(cubesql_context *ctx, int argc, cubesql_value **argv), void (*xFinal)(cubesql_context *ctx));
	int		(*csql_register_sql_collation) (void *pluginref, const char *name, int text_encoding, void *user_data, int (*xCompare)(void *user_data, int len1, const void *str1, int len2, const void *str2), void(*xDestroy)(void *user_data));
	int		(*csql_register_command) (void *pluginref, const char *command, int enabled, int dbref_required, int context, int commandID, int privilegeCode, int (*xCommand)(cubesql_client *client, const char **args, int commandID));
	int		(*csql_register_protocol) (void *pluginref, int command, int bitfield, int (*xProtocol)(cubesql_client *client));
	
	// utils (16)
	int		(*csql_file_copy) (char *dest, char *src);
	int		(*csql_file_delete) (char *path);
	int		(*csql_file_exists) (const char *path);
	int		(*csql_file_rename) (const char *oldname, const char *newname);
	int		(*csql_file_write) (char *file_path, char *buffer, int len);
	int		(*csql_path_is_directory) (char *path);
	void	(*csql_path_build) (char *path, char *path1, const char *path2);
	int		(*csql_directory_create) (char *path);
	DIRREF	(*csql_dir_init) (char *dirpath);
	char*	(*csql_dir_read) (DIRREF dref);
	void	(*csql_log) (char *msg);
	int		(*csql_num_processors) (void);
	char*	(*csql_ip_address) (void);
	char*	(*csql_os_version) (char *buf, int len);
	char*	(*csql_compress_buffer) (char *buffer1, int buffer1Size, char *buffer2, int buffer2Size, int *compressedSize);
	void	(*csql_client_set_error) (cubesql_client *client, int errcode, char *errmsg);

	// preferences (3)
	char*	(*csql_pref_read) (char *key, char *domain, char *buffer, int blen);
	int		(*csql_pref_write) (char *key, char *value, char *domain);
	int		(*csql_pref_delete) (char *key, char *domain);
	
	// db (10)
	cubesql_db*(*csql_db_create_inmemory) (void);
	cubesql_db*(*csql_db_open) (char *name, char *key, int *flag);
	void	(*csql_db_close) (cubesql_db *db);
	int		(*csql_db_write) (cubesql_db *db, char *sql);
	int		(*csql_db_read_table) (cubesql_db *db, char *sql, char ***result, int *nrows, int *ncolumns);
	void	(*csql_db_free_table) (char **result);
	char*	(*csql_db_column_read) (cubesql_db *db, char *sql, int colnum, char *buffer, int blen);
	void	(*csql_db_send_cursor) (cubesql_db *db, char *sql, cubesql_client *client);
	int		(*csql_db_errcode) (cubesql_db *db);
	char*	(*csql_db_errmsg) (cubesql_db *db);
	
	// string (9)
	int		(*csql_strcmpi) (const char *s1, const char *s2);
	int		(*csql_strncmpi) (const char *s1, const char *s2, int i);
	char*	(*csql_strdup) (const char *s);
	char*	(*csql_strndup) (const char *s, size_t n);
	char*	(*csql_strtrim) (char *s);
	int		(*csql_strncasecmp) (const char *s1, const char *s2, size_t n);
	char*	(*csql_strcasestr) (const char *s, const char *find);
	char*	(*csql_strlowercase) (char *s);
	char*	(*csql_struppercase) (char *s);
	
	// sqlite 3 functions (32)
	void*	(*csql_aggregate_context) (cubesql_context*, int nBytes);
	void*	(*csql_user_data) (cubesql_context*);
	void*	(*csql_malloc) (int);
	void*	(*csql_realloc) (void*, int);
	void	(*csql_free) (void*);
	const void* (*csql_value_blob) (cubesql_value*);
	int		(*csql_value_bytes) (cubesql_value*);
	int		(*csql_value_bytes16) (cubesql_value*);
	double	(*csql_value_double) (cubesql_value*);
	int		(*csql_value_int) (cubesql_value*);
	cubesql_int64 (*csql_value_int64) (cubesql_value*);
	const unsigned char* (*csql_value_text) (cubesql_value*);
	const void* (*csql_value_text16) (cubesql_value*);
	const void* (*csql_value_text16le) (cubesql_value*);
	const void* (*csql_value_text16be) (cubesql_value*);
	int		(*csql_value_type) (cubesql_value*);
	int		(*csql_value_numeric_type) (cubesql_value*);
	void	(*csql_result_blob) (cubesql_context*, const void*, int, void(*)(void*));
	void	(*csql_result_double) (cubesql_context*, double);
	void	(*csql_result_error) (cubesql_context*, const char*, int);
	void	(*csql_result_error16) (cubesql_context*, const void*, int);
	void	(*csql_result_error_toobig) (cubesql_context*);
	void	(*csql_result_error_nomem) (cubesql_context*);
	void	(*csql_result_int) (cubesql_context*, int);
	void	(*csql_result_int64) (cubesql_context*, cubesql_int64);
	void	(*csql_result_null) (cubesql_context*);
	void	(*csql_result_text) (cubesql_context*, const char*, int, void(*)(void*));
	void	(*csql_result_text16) (cubesql_context*, const void*, int, void(*)(void*));
	void	(*csql_result_text16le) (cubesql_context*, const void*, int,void(*)(void*));
	void	(*csql_result_text16be) (cubesql_context*, const void*, int,void(*)(void*));
	void	(*csql_result_value) (cubesql_context*, cubesql_value*);
	void	(*csql_result_zeroblob) (cubesql_context*, int n);
	
	// NEW FUNCTIONS added 20110622 (YYYYMMDD)
	int		(*csql_client_userid) (void);
	char*	(*csql_client_groupid) (void);
	char*	(*csql_client_username) (void);
	char*	(*csql_client_groupname) (int);
	char*	(*csql_client_ip_address) (void);
	
	// NEW FUNCTION added 20130905
	int		(*csql_register_tokenizer) (void *pluginref, const char *name, const void *p, int plen);
	
} PluginResolver;

CUBESQL_API int  cubesql_plugin_main (PluginResolver *resolver);
CUBESQL_API void cubesql_plugin_init (void);
CUBESQL_API void cubesql_plugin_register (void *pluginref);
CUBESQL_API void cubesql_plugin_exit (void);
CUBESQL_API void cubesql_plugin_info (char **name, char **version, char **copyright, char **description);
CUBESQL_API int  cubesql_plugin_auth_connect (char *username, char *token, char *ip, char *timestamp);
CUBESQL_API int  cubesql_plugin_auth_database (char *username, char *token, char *ip, char *timestamp, char *dbname);
CUBESQL_API	int  cubesql_plugin_activate_extension (char ***extensions, char ***serials);

//  main functions (5)
int		cubesql_register_sql_function  (void *pluginref, const char *name, int num_args, int text_encoding, void *user_data, void (*xFunc)(cubesql_context *ctx, int argc, cubesql_value **argv));
int		cubesql_register_sql_aggregate (void *pluginref, const char *name, int num_args, int text_encoding, void *user_data, void (*xStep)(cubesql_context *ctx, int argc, cubesql_value **argv), void (*xFinal)(cubesql_context *ctx));
int		cubesql_register_sql_collation (void *pluginref, const char *name, int text_encoding, void *user_data, int (*xCompare)(void *user_data, int len1, const void *str1, int len2, const void *str2), void(*xDestroy)(void *user_data));
int		cubesql_register_command (void *pluginref, const char *command, int invisible, int dbref_required, int context, int commandID, int privilegeCode, int (*xCommand)(cubesql_client *client, const char **args, int commandID));
int		cubesql_register_protocol (void *pluginref, int command, int bitfield, int (*xProtocol)(cubesql_client *client));

// utils (16)
int		cubesql_file_copy (char *dest, char *src);
int		cubesql_file_delete (char *path);
int		cubesql_file_exists (const char *path);
int		cubesql_file_rename (const char *oldname, const char *newname);
int		cubesql_file_write (char *file_path, char *buffer, int len);
int		cubesql_path_is_directory (char *path);
void	cubesql_path_build (char *path, char *path1, const char *path2);
int		cubesql_directory_create (char *path);
DIRREF	cubesql_dir_init (char *dirpath);
char*	cubesql_dir_read (DIRREF dref);
void	cubesql_log (char *msg);
int		cubesql_num_processors (void);
char*	cubesql_ip_address (void);
char*	cubesql_os_version (char *buf, int len);
char*	cubesql_compress_buffer (char *buffer1, int buffer1Size, char *buffer2, int buffer2Size, int *compressedSize);
void	cubesql_client_set_error (cubesql_client *client, int errcode, char *errmsg);
	
// preferences (3)
char*	cubesql_pref_read (char *key, char *domain, char *buffer, int blen);
int		cubesql_pref_write (char *key, char *value, char *domain);
int		cubesql_pref_delete (char *key, char *domain);

// db (10)
cubesql_db* cubesql_db_create_inmemory (void);
cubesql_db* cubesql_db_open (char *name, char *key, int *flag);
void	cubesql_db_close (cubesql_db *db);
int		cubesql_db_write (cubesql_db *db, char *sql);
int		cubesql_db_read_table (cubesql_db *db, char *sql, char ***result, int *nrows, int *ncolumns);
void	cubesql_db_free_table (char **result);
char*	cubesql_db_column_read (cubesql_db *db, char *sql, int colnum, char *buffer, int blen);
void	cubesql_db_send_cursor (cubesql_db *db, char *sql, cubesql_client *client);
int		cubesql_db_errcode(cubesql_db *db);
char*	cubesql_db_errmsg (cubesql_db *db);
	
// string (9)
int		cubesql_strcmpi (const char *s1, const char *s2);
int		cubesql_strncmpi (const char *s1, const char *s2, int i);
char*	cubesql_strdup (const char *s);
char*	cubesql_strndup (const char *s, size_t n);
char*	cubesql_strtrim (char *s);
int		cubesql_strncasecmp (const char *s1, const char *s2, size_t n);
char*	cubesql_strcasestr (const char *s, const char *find);
char*	cubesql_strlowercase (char *s);
char*	cubesql_struppercase (char *s);
	
// sql (32)
void	*cubesql_user_data (cubesql_context *ctx);
void	*cubesql_aggregate_context (cubesql_context*, int nbytes);
void	*cubesql_malloc (int);
void	*cubesql_realloc (void*, int);
void	cubesql_free (void*);
const void *cubesql_value_blob (cubesql_value*);
int		cubesql_value_bytes (cubesql_value*);
int		cubesql_value_bytes16 (cubesql_value*);
double	cubesql_value_double (cubesql_value*);
int		cubesql_value_int (cubesql_value*);
cubesql_int64 cubesql_value_int64 (cubesql_value*);
const unsigned char *cubesql_value_text (cubesql_value*);
const void *cubesql_value_text16 (cubesql_value*);
const void *cubesql_value_text16le (cubesql_value*);
const void *cubesql_value_text16be (cubesql_value*);
int		cubesql_value_type (cubesql_value*);
int		cubesql_value_numeric_type (cubesql_value*);
void	cubesql_result_blob (cubesql_context*, const void*, int, void(*)(void*));
void	cubesql_result_double (cubesql_context*, double);
void	cubesql_result_error (cubesql_context*, const char*, int);
void	cubesql_result_error16 (cubesql_context*, const void*, int);
void	cubesql_result_error_toobig (cubesql_context*);
void	cubesql_result_error_nomem (cubesql_context*);
void	cubesql_result_int (cubesql_context*, int);
void	cubesql_result_int64 (cubesql_context*, cubesql_int64);
void	cubesql_result_null (cubesql_context*);
void	cubesql_result_text (cubesql_context*, const char*, int, void(*)(void*));
void	cubesql_result_text16 (cubesql_context*, const void*, int, void(*)(void*));
void	cubesql_result_text16le (cubesql_context*, const void*, int,void(*)(void*));
void	cubesql_result_text16be (cubesql_context*, const void*, int,void(*)(void*));
void	cubesql_result_value (cubesql_context*, cubesql_value*);
void	cubesql_result_zeroblob (cubesql_context*, int n);

// NEW FUNCTIONS added 20110622 (YYYYMMDD)
int		cubesql_client_userid (void);
char*	cubesql_client_groupid (void);
char*	cubesql_client_username (void);
char*	cubesql_client_groupname (int groupid);
char*	cubesql_client_ip_address (void);

// NEW FUNCTION added 20130905
int		cubesql_register_tokenizer (void *pluginref, const char *name, const void *p, int plen);

#ifdef __cplusplus
}
#endif

#endif
